<?php


/**
 * runs the given black box test against the given files
 * @param string $mainFileNameWithExtension the main file to compile (with extension)
 * @param $test
 * @param $fullWorkingDirPath
 * @param int|string $timeout the timeout in MS  (can be string from db or int from default values)
 * @param int|string $memoryLimit the memory limit in kb  (can be string from db or int from default values)
 * @param int|string $diskSpaceLimit the max disk space the program can write to (can be string from db or int from default values)
 * @param string $compileCmd the command to execute to compile the file(s)
 * @param string $execCmd the command to execute to run the test
 * @param $sourceFileExtensions array the list with the file extensions from the source files for the p language
 * @param $needCompilation bool true: should compile sources, false: not (this assumes the sources are already compiled)
 * @param $maxLinesToRead int the max lines to read from the user program (passed to the runner)
 * @param $maxErrLinesToRead int the max lines to read from the user program (from stderr) (passed to the runner)
 * @param $maxLinesToWrite int the max lines to write the user program (passed to the runner)
 * @param $uniqueSessionId string the unique session id for the test runner to handle sandbox file system management
 * @param $showTestRunnerDebugOutput 1: show internal debug output from the test runner (included in the test response)
 * @return array the result
 * @internal param array $allFiles all files to copy in teh dir to use
 * Format: array[int] => {fileName: string, fileContent: string}
 */
function do_compareFilesTest($mainFileNameWithExtension, $test, $fullWorkingDirPath,
                             $timeout, $memoryLimit, $diskSpaceLimit,
                             $compileCmd, $execCmd, $sourceFileExtensions, $needCompilation,
                             $maxLinesToRead, $maxErrLinesToRead, $maxLinesToWrite,
                             $uniqueSessionId,
                             $showTestRunnerDebugOutput


)
{

    global $isDebug;
    global $config;
    global $s_command_compareFilesTest;
    global $s_fullWorkingDirPath;
    global $status_code_FileSystemIssue;
    global $s_test_id;
    global $s_test_content;
    global $s_passed;
    global $s_hasCompiled;
    global $s_programExitCode;
    global $s_testResultCode;
    global $s_protocol;
    global $responseNewLineString;
    global $s_test_allAssets;
    global $s_return_val;
    global $s_user_program_exit_code;
    global $s_output;
    global $s_fileName;
    global $s_fileHash;


    $command_to_execute_string = $s_command_compareFilesTest;


    # compile is done in the runner
    $commandToExecute = $execCmd;
    $output = [];
    $return_var = -1;

    $testContent = $test[$s_test_content];
    $allAssets = $test[$s_test_allAssets];

    # create the assets for the current test
    if (createTestAssets($allAssets, $fullWorkingDirPath) === FALSE) {
        //error message is in createFiles function
        return array(
            $s_return_val => 100,
            $s_output => 'could not create asset(s)',
            $s_user_program_exit_code => 0
        );
    }

    $fileHashInfo = "";
    for ($i = 0; $i < count($allAssets); $i++) {
        $file = $allAssets[$i];

        if ($i === count($allAssets) - 1) {
            $fileHashInfo = $fileHashInfo . $file[$s_fileName] . ":" . $file[$s_fileHash];
            continue;
        }

        $fileHashInfo = $fileHashInfo . $file[$s_fileName] . ":" . $file[$s_fileHash] . " ";
    }

    $longCmd = $config['runner']
        . ' ' . $command_to_execute_string                           # arg[0] the test method
        . ' "' . $fullWorkingDirPath . '"'                           # arg[1] dir path
        . ' ' . $mainFileNameWithExtension                           # arg[2] file path
        . ' "' . $compileCmd . '"'                                   # arg[3] command to execute # currently ignored because we call compile in php...
        . ' "' . $commandToExecute . '"'                             # arg[4] command to execute the test
        . ' ' . $timeout                                             # arg[5] the timeout
        . ' ' . $memoryLimit                                         # arg[6] the memory limit
        . ' ' . $diskSpaceLimit                                      # arg[7] the disk limit
        . ' "' . implode(',', $sourceFileExtensions) . '"'     # arg[8] the source file extensions
        . ' ' . ($needCompilation ? 'true' : 'false')                # arg[9] true: compile program new, false: not
        . ' "' . $maxLinesToRead . '"'                               # arg[10] max lines to read from the user program (inclusive)
        . ' "' . $maxErrLinesToRead . '"'                            # arg[11] max lines to read from the user program stderr (inclusive)
        . ' "' . $maxLinesToWrite . '"'                              # arg[12]  max lines to write to the user program (inclusive)
        . ' "' . $uniqueSessionId . '"'                              # arg[13] a unique session id
        . ' "' . ($showTestRunnerDebugOutput === TRUE ? 1 : 0) . '"' # arg[14] showTestRunnerDebugOutput
        . ' "' . $fileHashInfo . '"'                                  # arg[15] file:hash list

    ;

    if ($isDebug) {
        debug("using runner: " . $config['runner']);
        debug("full command: " . $longCmd);
    }

    # the test content is passed in as stdin (because might be too long for a console argument
    # e.g. windows its 32bit ca. 2.000 kb...

    $pipesDescriptor = array(
        0 => array('pipe', 'r'), # stdin is a pipe that the child will read from
        1 => array('pipe', 'w'),  # stdout is a pipe that the child will write to
        2 => array("pipe", "w")   # stderr
    );


    if ($isDebug) {
        $time_pre = microtime(true);
    }


    $env = $config['environmentVarsParsed'];

    # without bypass_shell it won't work on windows
    $process = proc_open($longCmd, $pipesDescriptor, $pipes, $fullWorkingDirPath, $env, array('bypass_shell' => TRUE));

//    $state = proc_get_status($process);
//    warn('open pid: ' . $state['pid']);

    if (is_resource($process)) {
        fwrite($pipes[0], $testContent);
        fclose($pipes[0]);


        $output = stream_get_contents($pipes[1]);
        fclose($pipes[1]);

        $errorOutput = stream_get_contents($pipes[2]);
        fclose($pipes[2]);

        $return_var = proc_close($process);
    }

    # we cannot use exec because this does no let us write to the stdin of the process
    #exec($longCmd, $output, $return_var);

    if ($isDebug) {
        $time_post = microtime(true);
        $exec_time = ($time_post - $time_pre) * 1000; //in ms
        debug("time to run the black-box-tests: " . $exec_time);
    }

    if ($isDebug && (isset($errorOutput) && trim($errorOutput) !== '')) {
        debug("error during execution of the (blackbox) test runner: " . $errorOutput);
    }


    # the test assets are removed when the dir gets removed
    # also the test runner could renamed/added/deleted some files

    //extract the program exit code from the protocol if any/possible else pretend 0
    $userProgramExitCode = 0;
    //the test runner uses \n
    $maybeExitOutput = substr($output, 0, strpos($output, "\n"));

    if (isset($maybeExitOutput)) {
        $exitCodeOutputPrefix = 'exit:';

        $checkString = substr($maybeExitOutput, 0, strlen($exitCodeOutputPrefix));

        //user output as > as start
        if ($checkString === $exitCodeOutputPrefix) {
            $exitCodeString = substr($maybeExitOutput, strlen($exitCodeOutputPrefix));

            $userProgramExitCode = intval($exitCodeString, 10);
        }
    }

    //$output

    #file_put_contents($fullWorkingDirPath . 'xyz.txt', $output);

    return array(
        $s_return_val => $return_var,
        $s_output => $output,
        $s_user_program_exit_code => $userProgramExitCode
    );
}
